@@ -31,7 +31,6 @@ bmo_2014.to_csv('data/bmo_2014.csv')
 import collections
 import csv
 import datetime
-import itertools
 import json
 import os
 import sys
@@ -214,6 +213,54 @@ class Client(object):
         response = req.json()
         return response[0]
 
+    def list_emploistore_services(self):
+        """List all the user-facing services proposed by the Emploi Store.
+
+        See the user-facing website at:
+            https://www.emploi-store.fr/portail/accueil
+
+        Returns:
+            a list of dicts, see
+            https://www.emploi-store-dev.fr/portail-developpeur-cms/home/catalogue-des-api/documentation-des-api/api-catalogueemploistore-v1/recuperer-les-services.html
+            for details of the fields. It includes "identifiantService" that
+            you can use for describe_emploistore_service.
+        """
+        scope = 'api_cataloguedesservicesemploistorev1 emploistoreusagers'
+        req = requests.get(
+            self.api_url + '/cataloguedesservicesemploistore/v1/api-emploistore/fichesservices',
+            headers={'Authorization': 'Bearer %s' % self.access_token(scope)})
+        req.raise_for_status()
+        return req.json()
+
+    def describe_emploistore_service(self, service_id, should_get_images=False):
+        """Describe one of the service of the Emploi Store.
+
+        See the user-facing website at:
+            https://www.emploi-store.fr/portail/accueil
+
+        Args:
+            service_id: the unique ID of the service to describe, see the
+                result of list_emploistore_services. It's also the last par of
+                the URL on the Emploi Store website: e.g.
+                https://www.emploi-store.fr/portail/services/sInformerSurLAlternance
+            should_get_images: whether to retrieve related images (logo,
+                screenshots, etc.). If True, the response will have a field
+                ressourcesFicheService containing the imags base64 encoded.
+
+        Returns:
+            a list of dicts, see
+            https://www.emploi-store-dev.fr/portail-developpeur-cms/home/catalogue-des-api/documentation-des-api/api-catalogueemploistore-v1/consulter-un-service.html
+            for details of the fields.
+        """
+        scope = 'api_cataloguedesservicesemploistorev1 emploistoreusagers'
+        req = requests.get(
+            self.api_url +
+            '/cataloguedesservicesemploistore/v1/api-emploistore/fichesservices/%s/%s'
+            % (service_id, 'true' if should_get_images else 'false'),
+            headers={'Authorization': 'Bearer %s' % self.access_token(scope)})
+        req.raise_for_status()
+        return req.json()
+
 
 class Package(object):
     """A package of resources available.
@@ -280,30 +327,30 @@ class Resource(object):
         res = self._client.api_get(
             '/datastore_search', limit=batch_size, offset=offset, id=self._id,
             filters=filters_json, fields=fields_list)
-        return res.get('records')
+        return res.get('total', 0), res.get('records')
 
     def records(self, batch_size=200, filters=None, fields=None):
         """Get all records from resource."""
-        offset = 0
-        while True:
-            batch = self._records_batch(offset, batch_size, filters, fields)
-            for record in batch:
-                yield record
-            if len(batch) != batch_size:
-                return
-            offset += batch_size
+        return _ResourceIterator(
+            lambda offset: self._records_batch(offset, batch_size, filters, fields),
+            batch_size)
 
-    def to_csv(self, file_name, fieldnames=None, batch_size=200, filters=None):
+    def to_csv(self, file_name, fieldnames=None, batch_size=200, filters=None, iterator=None):
         """Write all records to a CSV file.
 
-        If fieldnames isn't set, it will get them from the first record and
-        sort them alphabetically.
+        Args:
+            file_name: the path to the CSV file to create.
+            fieldnames: the list of fields to save. If not set, it will get
+                them from the first record and sort them alphabetically.
+            batch_size: the size of the batch of records to download.
+            filters: optional filters not to ask the whole resource.
+            iterator: a wrapper around the iterator on records, so that you can
+                modify the records or just keep track of progress.
         """
         records = self.records(batch_size=batch_size, filters=filters)
         need_utf8_encode = sys.version_info < (3, 0)
         if not fieldnames:
-            first = next(records)
-            records = itertools.chain([first], records)  # pylint: disable=redefined-variable-type
+            first = records.peek_first()
             keys = set(_strip_bom(k) for k in first.keys())
             fieldnames = sorted(keys - set(['_id']))
         if need_utf8_encode:
@@ -312,7 +359,7 @@ class Resource(object):
             csv_writer = csv.DictWriter(
                 csvfile, fieldnames, extrasaction='ignore')
             csv_writer.writeheader()
-            for record in records:
+            for record in iterator(records) if iterator else records:
                 record = {_strip_bom(k): v for k, v in record.items()}
                 if need_utf8_encode:
                     record = {
@@ -321,6 +368,56 @@ class Resource(object):
                 csv_writer.writerow(record)
 
 
+class _ResourceIterator(object):
+
+    def __init__(self, get_batch, batch_size):
+        self._get_batch = get_batch
+        self._batch_size = batch_size
+
+        self._offset = 0
+        self._generator = self._create_generator()
+
+        self._num_records = 0
+        self._first_batch = None
+
+    def _ensure_first_batch(self):
+        if self._first_batch is None:
+            self._num_records, self._first_batch = self._get_batch(0)
+        return self._num_records, self._first_batch
+
+    def __iter__(self):
+        return self
+
+    def __next__(self):
+        return next(self._generator)
+
+    # For Python 2 compatibility.
+    def next(self):
+        return next(self._generator)
+
+    def _create_generator(self):
+        offset = 0
+        while True:
+            if offset == 0:
+                _, batch = self._ensure_first_batch()
+            else:
+                _, batch = self._get_batch(offset)
+            for record in batch:
+                yield record
+            if len(batch) != self._batch_size:
+                return
+            offset += self._batch_size
+
+    def __len__(self):
+        self._ensure_first_batch()
+        return self._num_records
+
+    def peek_first(self):
+        """Peek on the first record without consuming it."""
+        _, first_batch = self._ensure_first_batch()
+        return first_batch[0]
+
+
 def _strip_bom(field):
     if field.startswith(_BOM):
         return field[len(_BOM):]
